/**
 * Zepto 简单的懒加载，基于Zepto
 * threshold{int} 用于控制提前加载的高度
 * placeholder{string} 可以是图片路径，也可以是64位编码的图片内容，未加载成功前显示的图片
 * 使用方式:$("DOM").picLazyLoad(threshold,placeholder);
 */

;
(function($) {
	$.fn.picLazyLoad = function(settings) {
		var debug = false; // 默认打印调试日志
		var $this = $(this),
			_winScrollTop = 0,
			_winHeight = $(window).height();

		settings = $.extend({
			threshold: 0, // 提前高度加载
			placeholder: ''
		}, settings || {});

		// 执行懒加载图片
		lazyLoadPic();

		// 滚动触发换图
		$(window).on('scroll', function() {
			_winScrollTop = $(window).scrollTop();
			lazyLoadPic();
		});

		// 清理缓存
		plus.io.requestFileSystem(plus.io.PUBLIC_DOWNLOADS, function(fs) {
			fs.root.getDirectory('_downloads/lzimage/', {
				create: true
			}, function(dir) {
				if (dir.isDirectory == true) {
					dir.getMetadata(function(metadata) {
						// 数据只能在该函数内部显示
						debug && console.log("原始文件大小：" + metadata.size / 1024 + "KB");
						if (parseInt(metadata.size / 1024 / 1024) > 50) {
							plus.io.resolveLocalFileSystemURL('_downloads/lzimage/', function(entry) {
								entry.removeRecursively(function(entry) {
									debug && console.log("临时文件删除成功");
								}, function(e) {
									debug && console.log("临时文件删除失败");
								});
							});
						}
					}, function(e) {
						//	alert("您的系统并非常规系统" + e);
					});
				}
			});
		});

		// 懒加载图片
		function lazyLoadPic() {
			$this.each(function() {
				var $self = $(this);
				// 如果是img
				if ($self.is('img')) {
					if ($self.attr('data-lazyload')) {
						var _offsetTop = $self.offset().top;
						if ((_offsetTop - settings.threshold) <= (_winHeight + _winScrollTop)) {
							CacheLoad(this); //暂时图片缓存只支持IMG
						}
					}
					// 如果是背景图
				} else {
					if ($self.attr('data-lazyload')) {
						// 默认占位图片
						if ($self.css('background-image') == 'none') {
							$self.css('background-image', 'url(' + settings.placeholder + ')');
						}
						var _offsetTop = $self.offset().top;
						if ((_offsetTop - settings.threshold) <= (_winHeight + _winScrollTop)) {
							$self.css('background-image', 'url(' + $self.attr('data-lazyload') + ')');
							$self.removeAttr('data-lazyload');
						}
					}
				}
			});
		}

		//缓存图片 用法 CacheLoad(this)
		function CacheLoad(obj, callback) {
			if (obj.getAttribute('data-loaded') == "ok") {
				return;
			}

			var image_url = obj.getAttribute('data-lazyload');
			debug && console.log(image_url);

			// 1. 转换网络图片地址为本地缓存图片路径，判断该图片是否存在本地缓存
			// http://...jpg -> md5
			// 缓存目录 _downloads/image/(md5).jpg
			var image_md5 = md5(image_url);
			var local_image_url = '_downloads/lzimage/' + image_md5 + '.jpg'; // 缓存本地图片url
			var absolute_image_path = plus.io.convertLocalFileSystemURL(local_image_url); // 平台绝对路径
			debug && console.log(absolute_image_path);

			// new temp_img 用于判断图片文件是否存在
			var temp_img = new Image();
			temp_img.src = absolute_image_path;
			temp_img.onload = function() {
				debug && console.log('存在本地缓存图片文件' + absolute_image_path + '，直接显示');
				// 1.1 存在，则直接显示（本地已缓存，不需要淡入动画）
				obj.setAttribute("src", absolute_image_path);
				obj.setAttribute("data-loaded", "ok");
				callback && callback();
				return;
			}
			temp_img.onerror = function() {
				debug && console.log('不存在本地缓存图片文件');
				// 1.2 不存在则进行加载图片
				var img = new Image();
				img.src = image_url; // 传过来的图片路径在这里用
				img.onload = function() {
					var that = this;
					obj.setAttribute("src", image_url);
					obj.setAttribute("data-loaded", "ok");
					// obj.classList.add('img-animation');
					callback && callback();

					// 1.3 下载图片缓存到本地
					debug && console.log('开始下载图片' + image_url + ' 缓存到本地: ' + local_image_url);
					var download_task = plus.downloader.createDownload(image_url, {
						filename: local_image_url // filename:下载任务在本地保存的文件路径
					}, function(download, status) {
						if (status != 200) {
							// 下载失败,删除本地临时文件
							debug && console.log('下载失败,status' + status);
							if (local_image_url != null) {
								plus.io.resolveLocalFileSystemURL(local_image_url, function(entry) {
									entry.remove(function(entry) {
										debug && console.log("临时文件删除成功" + local_image_url);
									}, function(e) {
										debug && console.log("临时文件删除失败" + local_image_url);
									});
								});
							}
						}
					});
					download_task.start();
				}
			}
		}
	}
})(Zepto);